/************************************************************************/
/* GPS LIB                                                              */
/* Ver 1.0																*/
/* Last Edited:1395/06/16                                               */
/* Abolfazl Shakeri                                                     */
/* Designer2013.blogsky.com                                             */
/************************************************************************/
#ifndef _GPS_LIB_H
#define _GPS_LIB_H

	#include <avr/interrupt.h>
	#include <util/delay.h>
	#include <string.h>
	#include "compiler.h"
	#include "uart_ATX.h"
/************************************************************************/
/* EDIT THIS DEFINE's                                                   */
/************************************************************************/
	//#define DEBUGE_GPS				1
	
	#define gps_select_uart			uartE1
	#define GPS_UART				&USARTE1
	#define GPS_UART_ISR			USARTE1_RXC_vect
	
#ifdef DEBUGE_GPS
	#define dbg_uart_gps			uartF0
	#define DBG_UART_GPS			&USARTF0
#endif
/************************************************************************/
/* DONT EDIT THIS DEFINE's                                              */
/************************************************************************/
	#define gps_enable				uart_interrupt_init(GPS_UART,low_level_interrupt)
	#define gps_disable				uart_interrupt_init(GPS_UART,disable_interrupt)

	#define longtitude_length		15
	#define latitude_length			15
	#define gps_length				60
	struct{
		char buffer[gps_length+1],latitude[latitude_length+1],longtitude[longtitude_length+1],ns,ew;
		int index;
		char len,valid;
	}gps;
	char buff_gps[15];
	
	void gps_init();
	void dm_to_dd(char dd_lat[], char dd_lon[]);
	void dm_to_degree(char de_lat[], char de_lon[]);
	void detect_gps();
	
	void gps_init()
	{
		uart_init(GPS_UART,gps_select_uart,baud_9600,conf_8_no_1);
		
		#ifdef DEBUGE_GPS
			uart_init(DBG_UART_GPS,dbg_uart_gps,baud_115200,conf_8_no_1);
		#endif
		/************************************************************************/
		/* ENABLE LEVEL INTERRUPT                                               */
		/************************************************************************/
		PMIC.CTRL |= PMIC_LOLVLEN_bm;
	}
	
	void dm_to_dd(char dd_lat[], char dd_lon[])
	{
		int z,dd;
		double dd2;
		
		gps.len = strlen(gps.latitude);
		memset(buff_gps,0,15);
		buff_gps[0]=gps.latitude[0];buff_gps[1]=gps.latitude[1];
		dd=atoi(buff_gps);
		for(z=2; z<=gps.len; z++){buff_gps[z-2]=gps.latitude[z];}
		dd2=atof(buff_gps);
		dd2/=60;
		if (dd>0)		dd2 = dd + dd2;
		else		dd2 = dd - dd2;
		memset(gps.latitude,0,latitude_length);
		sprintf(dd_lat,"%02.06f",dd2);
		
		gps.len = strlen(gps.longtitude);
		memset(buff_gps,0,15);
		buff_gps[0]=gps.longtitude[1];buff_gps[1]=gps.longtitude[2];
		dd=atoi(buff_gps);
		for(z=3; z<=gps.len; z++){buff_gps[z-3]=gps.longtitude[z];}
		dd2=atof(buff_gps);
		dd2/=60;
		if (dd>0)		dd2 = dd + dd2;
		else		dd2 = dd - dd2;
		memset(gps.longtitude,0,latitude_length);
		sprintf(dd_lon,"%02.06f",dd2);
	}
	
	void dm_to_degree(char de_lat[], char de_lon[])
	{
		int d,m;
		float f,ddd,s;
		
		dm_to_dd(gps.latitude,gps.longtitude);
		
		f = atof(gps.latitude);
		d = (int)f;
		ddd = (float)(f-d);
		m = (int)(ddd*60);
		s = (float)(m)/60;
		s = (float)(ddd) - (float)(s);
		s = (float)(s)*3600;
		sprintf(de_lat,"%d%d\'%02.01f\"",d,m,s);
		
		f = atof(gps.longtitude);
		d = (int)f;
		ddd = (float)(f-d);
		m = (int)(ddd*60);
		s = (float)(m)/60;
		s = (float)(ddd) - (float)(s);
		s = (float)(s)*3600;
		sprintf(de_lon,"%d%d\'%02.01f\"",d,m,s);
	}

	void detect_gps()
	{
		int g=0,g2=0,g3=0;
		if (strstr_P(gps.buffer,PSTR("$GPRMC"))!=0)
		{
			gps.len = strlen(gps.buffer);
			for (g=0; g<gps.len; g++)
			{
				if(gps.buffer[g]==',')g2++;
				if(g2==3)break;
			}
			if (g2==3)
			{
				memset(gps.latitude,0,latitude_length);
				memset(gps.longtitude,0,longtitude_length);
				
				g3=0;
				for (g2=g+1; g2<gps.len; g2++)
				{
					if(gps.buffer[g2]!=','){gps.latitude[g3]=gps.buffer[g2];g3++;}
					else if(gps.buffer[g2]==',')break;
				}
				
				gps.ns = gps.buffer[g2+1];
				
				g3=0;
				for (g=g2+3; g<gps.len; g++)
				{
					if(gps.buffer[g]!=','){gps.longtitude[g3]=gps.buffer[g];g3++;}
					else if(gps.buffer[g]==',')break;
				}
				
				gps.ew = gps.buffer[g+1];
				
				//dm_to_dd();
				
				gps.valid = ENABLE;
				
				#ifdef DEBUGE_GPS
					uart_put_string(gps.latitude,DBG_UART_GPS,0);
					uart_putchar(gps.ns,DBG_UART_GPS);
					uart_put_string(gps.longtitude,DBG_UART_GPS,0);
					uart_putchar(gps.ew,DBG_UART_GPS);
					uart_putchar(13,DBG_UART_GPS);
				#endif
			}	
		}	
	}
	
	ISR(GPS_UART_ISR)
	{
		gps.buffer[gps.index]=uart_getchar(GPS_UART);
		if (gps.buffer[gps.index]=='\r')
		{
			for (gps.index=gps.index; gps.index<gps_length; gps.index++)
			{
				gps.buffer[gps.index]=0;
			}
			gps.index = 0;
			#ifdef DEBUGE_GPS
			uart_put_string(gps.buffer,DBG_UART_GPS,1);
			#endif
			detect_gps();
		}
		else if(gps.index < gps_length-1){gps.index++;}
	}
#endif