//*******************************************************************************************
// DESIGNER2013.BLOGSKY.COM
// ABOLFAZL SHAKERI
// V.1
//
//*******************************************************************************************
#ifndef _UART_ATX_H
#define _UART_ATX_H

#include "compiler.h"
#include <stdarg.h>
#include <avr/io.h>

#define uartC0			1
#define uartC1			2
#define uartD0			3
#define uartD1			4
#define uartE0			5
#define uartE1			6
#define uartF0			7
#define uartF1			8

#define baud_9600		9600
#define baud_57600		57600
#define baud_19200		19200
#define baud_115200		115200
#define baud_2400		2400
#define baud_300		300

#define disable_interrupt		0
#define low_level_interrupt		1
#define medium_level_interrupt	2
#define high_level_interrupt	3

#define conf_8_no_1				USART_CHSIZE_8BIT_gc | USART_CMODE_ASYNCHRONOUS_gc | USART_PMODE_DISABLED_gc
#define conf_8_even_1			USART_CHSIZE_8BIT_gc | USART_CMODE_ASYNCHRONOUS_gc | USART_PMODE_EVEN_gc
#define conf_7_no_1				USART_CHSIZE_7BIT_gc | USART_CMODE_ASYNCHRONOUS_gc | USART_PMODE_DISABLED_gc
#define conf_7_even_1			USART_CHSIZE_7BIT_gc | USART_CMODE_ASYNCHRONOUS_gc | USART_PMODE_EVEN_gc
#define conf_7_odd_1			USART_CHSIZE_7BIT_gc | USART_CMODE_ASYNCHRONOUS_gc | USART_PMODE_ODD_gc

#define DATA_BUF_LEN			50

#define  BSEL(x)   ((F_CPU/(8*x))-1)	/* 2x clock */
#define  BAUDA(x)  ((BSEL(x)) & 0xff)
#define  BAUDB(x)  ((BSEL(x) >> 8) | 0)		/* BSCALE of 0 */

#define SERIAL_UBBRVAL(Baud)    ((((F_CPU / 16) + (Baud / 2)) / (Baud)) - 1)

int i;
char lo[3],dd=0;
int e_e=0;
U8 uart_getchar (USART_t *numuart);
void uart_disable(USART_t *numuart);
void uart_putchar (unsigned char ch,USART_t *numuart);
void uart_interrupt_init(USART_t *numuart, char intrrupt_mode);
void uart_put_string (char *data_string, USART_t *numuart, char enter);
void uart_init(USART_t *numuart,char uart_select,const uint32_t baud,char conf);
U8 uart_getstring (char *buffer_data, unsigned int len_buffer,USART_t *numuart, char is_esp);

//****************************************************************************************************
//
//UART INIT
//
//****************************************************************************************************
void uart_init(USART_t *numuart,char uart_select,const uint32_t baud,char conf)
{	
	switch(uart_select)
	{
		case uartC0:
		PORTC_OUTSET = PIN3_bm;
		PORTC_DIRSET = PIN3_bm;
		break;
		
		case uartC1:
		PORTC_OUTSET = PIN7_bm;
		PORTC_DIRSET = PIN7_bm;
		break;
		
		case uartD0:
		PORTD_OUTSET = PIN3_bm;
		PORTD_DIRSET = PIN3_bm;
		break;
		
		case uartD1:
		PORTD_OUTSET = PIN7_bm;
		PORTD_DIRSET = PIN7_bm;
		break;
		
		case uartE0:
		PORTE_OUTSET = PIN3_bm;
		PORTE_DIRSET = PIN3_bm;
		break;
		
#if defined(__AVR_ATxmega64A3__) \
		|| defined(__AVR_ATxmega64A3U__) \
		|| defined(__AVR_ATxmega128A3__) \
		|| defined(__AVR_ATxmega128A3A__) \
		|| defined(__AVR_ATxmega128A3U__) \
		|| defined(__AVR_ATxmega128A1U__)
		case uartE1:
		PORTE_OUTSET = PIN7_bm;
		PORTE_DIRSET = PIN7_bm;
		break;
		
		case uartF0:
		PORTF_OUTSET = PIN3_bm;
		PORTF_DIRSET = PIN3_bm;
		break;
		
		case uartF1:
		PORTF_OUTSET = PIN7_bm;
		PORTF_DIRSET = PIN7_bm;
		break;
#endif
	}
	
	numuart->CTRLA = 0;
	numuart->CTRLB = USART_RXEN_bm | USART_TXEN_bm;
	numuart->CTRLC = conf;
		
	/*numuart->BAUDCTRLB = BAUDA(baud);		// do this reg first!
	numuart->BAUDCTRLA = BAUDB(baud);		// set up the baud rate*/
	uint16_t BaudValue = SERIAL_UBBRVAL(baud);
	numuart->BAUDCTRLB = (BaudValue >> 8);
	numuart->BAUDCTRLA = (BaudValue & 0xFF);
}

//****************************************************************************************************
//
//UART DISABLE
//
//****************************************************************************************************
void uart_disable(USART_t *numuart)
{
	numuart->CTRLA = 0;
	numuart->CTRLB = 0;
	numuart->CTRLC = 0;
}

//***************************************************************************************************
//
// INTERRUPT OF UART INIT
//
//***************************************************************************************************
void uart_interrupt_init(USART_t *numuart, char intrrupt_mode)
{
	switch(intrrupt_mode)
	{
		case 0:
		numuart->CTRLA = USART_RXCINTLVL_OFF_gc;
		break;
		
		case 1:
		numuart->CTRLA = USART_RXCINTLVL_LO_gc;
		break;
		
		case 2:
		numuart->CTRLA = USART_RXCINTLVL_MED_gc;
		break;
		
		case 3:
		numuart->CTRLA = USART_RXCINTLVL_HI_gc;
		break;
	}
}
//------------------------------------------------------------------------------
//  @fn uart_getchar
//!
//! Get a character from the UART peripheral.
//!
//! @warning none
//!
//! @param  none
//!
//! @return read (received) character on the UART
//!
//------------------------------------------------------------------------------
U8 uart_getchar (USART_t *numuart)
{
	U8 ch=0;
	while(!(numuart->STATUS & USART_RXCIF_bm));
	ch = numuart->DATA;
	return ch;
}

//------------------------------------------------------------------------------
//  @fn uart_getstring
//!
//! Get a string from the UART peripheral.
//!
//! @return read (received) string on the UART
//!
//------------------------------------------------------------------------------
/*
U8 uart_getstring (char *buffer_data, unsigned int len_buffer,USART_t *numuart, char is_esp)
{
	U8 z=0;
	
	/ *for (i=0;i<len_buffer;i++)
	{
		buffer_data[i]=0;
	}
	i=0;* /
	
	//time = 1 sec
	//timer clock => 32000000 / per scaler->1024=31250
	//timer cycle => 1 / 31250 = 32 us
	//tcc1.per    => (200000us/32us) = 6250
	TCC1.CNT = 0;
	if(is_esp==ENABLE){TCC1.PER = 6250;}else{TCC1.PER = 31250;}
	TCC1.CTRLB = TC_WGMODE_NORMAL_gc;
	//TCC1.INTCTRLA = TC_OVFINTLVL_LO_gc;
	TCC1.CTRLA = TC_CLKSEL_DIV1024_gc;
	
	len_buffer -= 2;
	e_e = 0;
	dd = 0;
	lo[2]=0;
	lo[1]=0;
	i=0;
	
	while(1)
	{
		//if (i>len_buffer){TCC1.CTRLA = TC_CLKSEL_OFF_gc;return i;}
		
		//z = uart_getchar(numuart);
		switch(numuart)
		{
			case uartC0: while(!(USARTC0_STATUS & USART_RXCIF_bm)){if(TCC1.INTFLAGS & TC1_OVFIF_bm){TCC1.INTFLAGS = TC1_OVFIF_bm;i=1;z=0x0D;goto br;}};z = USARTC0_DATA;break;
			case uartC1: while(!(USARTC1_STATUS & USART_RXCIF_bm)){if(TCC1.INTFLAGS & TC1_OVFIF_bm){TCC1.INTFLAGS = TC1_OVFIF_bm;i=1;z=0x0D;goto br;}};z = USARTC1_DATA;break;
			
			case uartD0: while(!(USARTD0_STATUS & USART_RXCIF_bm)){if(TCC1.INTFLAGS & TC1_OVFIF_bm){TCC1.INTFLAGS = TC1_OVFIF_bm;i=1;z=0x0D;goto br;}};z = USARTD0_DATA;break;
			case uartD1: while(!(USARTD1_STATUS & USART_RXCIF_bm)){if(TCC1.INTFLAGS & TC1_OVFIF_bm){TCC1.INTFLAGS = TC1_OVFIF_bm;i=1;z=0x0D;goto br;}};z = USARTD1_DATA;break;
			
			case uartE0: while(!(USARTE0_STATUS & USART_RXCIF_bm)){if(TCC1.INTFLAGS & TC1_OVFIF_bm){TCC1.INTFLAGS = TC1_OVFIF_bm;i=1;z=0x0D;goto br;}};z = USARTE0_DATA;break;
		#if defined(__AVR_ATxmega64A3__) \
		|| defined(__AVR_ATxmega64A3U__) \
		|| defined(__AVR_ATxmega128A3__) \
		|| defined(__AVR_ATxmega128A3A__) \
		|| defined(__AVR_ATxmega128A3U__) \
		|| defined(__AVR_ATxmega128A1U__)
			case uartE1: while(!(USARTE1_STATUS & USART_RXCIF_bm)){if(TCC1.INTFLAGS & TC1_OVFIF_bm){TCC1.INTFLAGS = TC1_OVFIF_bm;i=1;z=0x0D;goto br;}};z = USARTE1_DATA;break;
			
			case uartF0: while(!(USARTF0_STATUS & USART_RXCIF_bm)){if(TCC1.INTFLAGS & TC1_OVFIF_bm){TCC1.INTFLAGS = TC1_OVFIF_bm;i=1;z=0x0D;goto br;}};z = USARTF0_DATA;break;
			case uartF1: while(!(USARTF1_STATUS & USART_RXCIF_bm)){if(TCC1.INTFLAGS & TC1_OVFIF_bm){TCC1.INTFLAGS = TC1_OVFIF_bm;i=1;z=0x0D;goto br;}};z = USARTF1_DATA;break;
		#endif
		}
		
		br:
		if ((z==0x0D/ * || z==0x0A* /) && (i != 0))
		{
			TCC1.CTRLA = 0;
			
			if (i>1)
			{
				for (i=i;i<len_buffer;i++)
				{
					buffer_data[i]=0;
				}
			}
			return i;
		}
		if (z!=0x0D && z!=0x0A && i<len_buffer){buffer_data[i] = z;i++;TCC1.CNT=0;}
			
		if (is_esp==ENABLE)
		{
			if (dd==3 && i==e_e)
			{
				TCC1.CTRLA = 0;
				return i;
			}
			
			if (dd==2)
			{
				if(buffer_data[(i-1)]!=':')
				{
					lo[e_e]=buffer_data[(i-1)]; e_e++;
				}
				else if(buffer_data[(i-1)]==':')
				{
					e_e = atoi(lo) + i;
					dd=3;
				}
			}
			
			if (buffer_data[0]=='+' && buffer_data[1]=='I' && buffer_data[2]=='P' && buffer_data[3]=='D' && buffer_data[(i-1)]==',' && dd<3)
			{
				dd++;
			}
		}
	}
}

*/
//------------------------------------------------------------------------------
//  @fn uart_putchar
//!
//! Send a character on the UART peripheral.
//!
//!
//! @param  character to send
//!
//! @return character sent
//!
//------------------------------------------------------------------------------
void uart_putchar (unsigned char ch,USART_t *numuart)
{
	while(!(numuart->STATUS & USART_DREIF_bm));
	numuart->DATA = ch;
}

//------------------------------------------------------------------------------
//  @fn uart_put_string
//!
//! Put a data-string on TX UART. The data-string is send up to null
//! character is found.
//!
//! @warning "uart_init()" must be performed before
//!
//! @param Pointer on U8 data-string
//!
//! @return (none)
//!
//------------------------------------------------------------------------------
void uart_put_string (char *data_string, USART_t *numuart, char enter)
{
	while(*data_string) uart_putchar (*data_string++,numuart);
	
	if (enter!=0){uart_putchar('\r',numuart);uart_putchar('\n',numuart);}
}

//-------------------------------------------------------------------------------
U8 uart_mini_printf(USART_t *numuart,char *format, ...)
{
	va_list arg_ptr;
	U8      *p,*sval;
	U8      u8_temp, n_sign, data_idx, min_size;
	char    data_buf[DATA_BUF_LEN];
	S8      long_flag, alt_p_c;
	S8      s8_val;
	S16     s16_val;
	S32     s32_val;
	U16     u16_val;
	U32     u32_val;

	long_flag = FALSE;
	alt_p_c = FALSE;
	min_size = DATA_BUF_LEN-1;

	va_start(arg_ptr, format);   // make arg_ptr point to the first unnamed arg
	for (p = (U8 *) format; *p; p++)
	{
		if ((*p == '%') || (alt_p_c == TRUE))
		{
			p++;
		}
		else
		{
			uart_putchar(*p,numuart);
			alt_p_c = FALSE;
			long_flag = FALSE;
			continue;   // "switch (*p)" section skipped
		}
		switch (*p)
		{
			case 'c':
			if (long_flag == TRUE)      // ERROR: 'l' before any 'c'
			{
				uart_putchar('l',numuart);
				uart_putchar('c',numuart);
			}
			else
			{
				s8_val = (S8)(va_arg(arg_ptr, int));    // s8_val = (S8)(va_arg(arg_ptr, S16));
				uart_putchar((U8)(s8_val),numuart);
			}
			// Clean up
			min_size = DATA_BUF_LEN-1;
			alt_p_c = FALSE;
			long_flag = FALSE;
			break; // case 'c'
			
			case 's':
			if (long_flag == TRUE)      // ERROR: 'l' before any 's'
			{
				uart_putchar('l',numuart);
				uart_putchar('s',numuart);
			}
			else
			{
				for (sval = va_arg(arg_ptr, U8 *); *sval; sval++)
				{
					uart_putchar(*sval,numuart);
				}
			}
			// Clean up
			min_size = DATA_BUF_LEN-1;
			alt_p_c = FALSE;
			long_flag = FALSE;
			break;  // case 's'
			
			case 'l':  // It is not the number "ONE" but the lower case of "L" character
			if (long_flag == TRUE)      // ERROR: two consecutive 'l'
			{
				uart_putchar('l',numuart);
				alt_p_c = FALSE;
				long_flag = FALSE;
			}
			else
			{
				alt_p_c = TRUE;
				long_flag = TRUE;
			}
			p--;
			break;  // case 'l'
			
			case 'd':
			n_sign  = FALSE;
			for(data_idx = 0; data_idx < (DATA_BUF_LEN-1); data_idx++)
			{
				data_buf[data_idx] = '0';
			}
			data_buf[DATA_BUF_LEN-1] = 0;
			data_idx = DATA_BUF_LEN - 2;
			if (long_flag)  // 32-bit
			{
				s32_val = va_arg(arg_ptr, S32);
				if (s32_val < 0)
				{
					n_sign = TRUE;
					s32_val  = -s32_val;
				}
				while (1)
				{
					data_buf[data_idx] = s32_val % 10 + '0';
					s32_val /= 10;
					data_idx--;
					if (s32_val==0) break;
				}
			}
			else  // 16-bit
			{
				s16_val = (S16)(va_arg(arg_ptr, int)); // s16_val = va_arg(arg_ptr, S16);
				if (s16_val < 0)
				{
					n_sign = TRUE;
					s16_val  = -s16_val;
				}
				while (1)
				{
					data_buf[data_idx] = s16_val % 10 + '0';
					s16_val /= 10;
					data_idx--;
					if (s16_val==0) break;
				}
			}
			if (n_sign) { uart_putchar('-',numuart); }
			data_idx++;
			if (min_size < data_idx)
			{
				data_idx = min_size;
			}
			uart_put_string (data_buf + data_idx,numuart,0);
			// Clean up
			min_size = DATA_BUF_LEN-1;
			alt_p_c = FALSE;
			long_flag = FALSE;
			break;  // case 'd'
			
			case 'u':
			for(data_idx = 0; data_idx < (DATA_BUF_LEN-1); data_idx++)
			{
				data_buf[data_idx] = '0';
			}
			data_buf[DATA_BUF_LEN-1] = 0;
			data_idx = DATA_BUF_LEN - 2;
			if (long_flag)  // 32-bit
			{
				u32_val = va_arg(arg_ptr, U32);
				while (1)
				{
					data_buf[data_idx] = u32_val % 10 + '0';
					u32_val /= 10;
					data_idx--;
					if (u32_val==0) break;
				}
			}
			else  // 16-bit
			{
				u16_val = (U16)(va_arg(arg_ptr, int)); // u16_val = va_arg(arg_ptr, U16);
				while (1)
				{
					data_buf[data_idx] = u16_val % 10 + '0';
					data_idx--;
					u16_val /= 10;
					if (u16_val==0) break;
				}
			}
			data_idx++;
			if (min_size < data_idx)
			{
				data_idx = min_size;
			}
			uart_put_string (data_buf + data_idx,numuart,0);
			// Clean up
			min_size = DATA_BUF_LEN-1;
			alt_p_c = FALSE;
			long_flag = FALSE;
			break;  // case 'u':
			
			case 'x':
			case 'X':
			for(data_idx = 0; data_idx < (DATA_BUF_LEN-1); data_idx++)
			{
				data_buf[data_idx] = '0';
			}
			data_buf[DATA_BUF_LEN-1] = 0;
			data_idx = DATA_BUF_LEN - 2;
			if (long_flag)  // 32-bit
			{
				u32_val = va_arg(arg_ptr, U32);
				while (u32_val)
				{
					u8_temp = (U8)(u32_val & 0x0F);
					data_buf[data_idx] = (u8_temp < 10)? u8_temp+'0':u8_temp-10+(*p=='x'?'a':'A');
					u32_val >>= 4;
					data_idx--;
				}
			}
			else  // 16-bit
			{
				u16_val = (U16)(va_arg(arg_ptr, int)); // u16_val = va_arg(arg_ptr, U16);
				while (u16_val)
				{
					u8_temp = (U8)(u16_val & 0x0F);
					data_buf[data_idx] = (u8_temp < 10)? u8_temp+'0':u8_temp-10+(*p=='x'?'a':'A');
					u16_val >>= 4;
					data_idx--;
				}
			}
			data_idx++;
			if (min_size < data_idx)
			{
				data_idx = min_size;
			}
			uart_put_string (data_buf + data_idx,numuart,0);
			// Clean up
			min_size = DATA_BUF_LEN-1;
			alt_p_c = FALSE;
			long_flag = FALSE;
			break;  // case 'x' & 'X'
			
			case '0':   // Max allowed "min_size" 2 decimal digit, truncated to DATA_BUF_LEN-1.
			min_size = DATA_BUF_LEN-1;
			if (long_flag == TRUE)      // ERROR: 'l' before '0'
			{
				uart_putchar('l',numuart);
				uart_putchar('0',numuart);
				// Clean up
				alt_p_c = FALSE;
				long_flag = FALSE;
				break;
			}
			u8_temp = *++p;
			if ((u8_temp >='0') && (u8_temp <='9'))
			{
				min_size = u8_temp & 0x0F;
				u8_temp = *++p;
				if ((u8_temp >='0') && (u8_temp <='9'))
				{
					min_size <<= 4;
					min_size |= (u8_temp & 0x0F);
					p++;
				}
				min_size = ((min_size & 0x0F) + ((min_size >> 4) *10));  // Decimal to hexa
				if (min_size > (DATA_BUF_LEN-1))
				{
					min_size = (DATA_BUF_LEN-1);
				}  // Truncation
				min_size = DATA_BUF_LEN-1 - min_size;  // "min_size" formatted as "data_ix"
			}
			else      // ERROR: any "char" after '0'
			{
				uart_putchar('0',numuart);
				uart_putchar(*p,numuart);
				// Clean up
				alt_p_c = FALSE;
				long_flag = FALSE;
				break;
			}
			p-=2;
			alt_p_c = TRUE;
			// Clean up
			long_flag = FALSE;
			break;  // case '0'
			
			default:
			if (long_flag == TRUE)
			{
				uart_putchar('l',numuart);
			}
			uart_putchar(*p,numuart);
			// Clean up
			min_size = DATA_BUF_LEN-1;
			alt_p_c = FALSE;
			long_flag = FALSE;
			break;  // default
			
		}   // switch (*p ...
		
	}   // for (p = ...
	
	va_end(arg_ptr);
	return 0;
}

#endif